#include <assert.h>
#include <limits.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

static const char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";

bool baseconv(char outbuf[], size_t bufsize, unsigned base, uintmax_t num) {
	// Supported bases are 2 to 36 inclusive (though it could be higher
	// if our digits string was longer
	if (base < 2 || base > 36)
		return false;

	// Longest possible converted string is base-2, where we produce one
	// character per bit of input, so if we size the buffer to fit it we
	// can fit any converted string
	char buf[sizeof(uintmax_t) * CHAR_BIT];

	// Place the digits in the buffer in a little-endian order
	size_t len = 0;
	while (num) {
		char digit = digits[num % base];
		buf[len++] = digit;
		num /= base;
	}

	// Special case: if the original input was zero, previous loop ran
	// zero times, and there is nothing in the buffer. Place a single
	// zero there.
	if (!len)
		buf[len++] = digits[0];

	// Can the output buffer hold the converted string as well as the
	// null terminator?
	if (!bufsize || bufsize - 1 < len)
		return false;

	// Copy the digits to the output buffer, reversing the order
	for (size_t i = 0; i < len; i++)
		outbuf[i] = buf[len - 1 - i];

	// Null-terminate
	outbuf[len] = '\0';

	return true;
}
